package ejb;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.jms.JMSContext;
import javax.jms.Queue;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import jpa.ParteJPA;
import jpa.UsuarioJPA;

@Stateless
public class DTFacadeBeanLocal implements DTFacadeLocal {

	@PersistenceContext(unitName = "primary")
	private EntityManager entman;
	
	@Resource(mappedName = "java:/queue/test")
	private Queue testQueue;

	@Inject
	private JMSContext jmsContext;
	
	/**
	 * Mètode que comproba si hi ha algún usuari d'oficina que es correspongui amb el login i el password d'entrada.
	 */
	@Override
	public UsuarioJPA autenticarUsuario(String login, String pass){
		
		UsuarioJPA usuario = null;
		try {
			usuario = (UsuarioJPA) entman.createQuery("from UsuarioJPA u where u.rol = 'oficina' and u.login = :login and u.password = :pass").setParameter("login", login).setParameter("pass", pass).getSingleResult();
		} catch (Exception e) {
		}
		return usuario;
	}

	/**
	 * Mètode que retorna una llista amb els usuaris transportistes disponibles al sistema.
	 */
	@Override
	public List<UsuarioJPA> verUsuarios() {
		
		@SuppressWarnings("unchecked")
//		Set<UsuarioJPA> allUsuarios = new HashSet<>(
//				entman.createQuery("from UsuarioJPA u where u.rol = 'transportista' order by u.nombre").getResultList());
		List<UsuarioJPA> allUsuarios = entman.createQuery("from UsuarioJPA u where u.rol = 'transportista' order by u.nombre").getResultList();
		return allUsuarios;
	}

	/**
	 * Mètode que retorna una llista dels comunicats de treball de l'usuari amb l'id d'entrada.
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Set<ParteJPA> verPartesPorUsuario(int id) {

		Set<ParteJPA> allPartesByUsuario = null;
		allPartesByUsuario = new HashSet<>(entman.createQuery("from ParteJPA p LEFT OUTER JOIN FETCH p.actividades a where p.conductor.id = :id")
				.setParameter("id", id).getResultList());
		return allPartesByUsuario;
	}

	/**
	 * Mètode que retorna un objecte ParteJPA amb totes les seves entitats relacionades per l'id d'entrada.
	 */
	@Override
	public ParteJPA verParte(int id) {
		
		ParteJPA parte = null;
		try {
			parte = (ParteJPA) entman
					.createQuery(
							"from ParteJPA p LEFT OUTER JOIN FETCH p.actividades a LEFT OUTER JOIN FETCH a.listas l LEFT OUTER JOIN FETCH l.fases f LEFT OUTER JOIN FETCH f.requisitos r where p.id = :id")
					.setParameter("id", id).getSingleResult();
		} catch (Exception e) {
		}
		return parte;
	}

	/**
	 * Mètode que envia un comunicat en format JSON a una cua JMS del servidor per la seva impressió en PDF al disc.
	 */
	@Override
	public void generarPdf(String json) {

		jmsContext.createProducer().send(testQueue, json);
	}
}
