package jpa;

import java.io.Serializable;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;

import com.fasterxml.jackson.annotation.JsonIgnore;

/**
 * Classe que implementa una activitat de un comunicat de treball.
 * @author Guillem
 *
 */
@Entity
@Table(name = "tfg.actividades")
public class ActividadJPA implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private int id;

	@ManyToOne(optional = false)
	@JoinColumn(name = "tipo")
	private TipoActividadJPA tipo;

	private Boolean abierta = true;
	private String transporte = "";
	private String vehiculo = "";
	private String remolque = "";
	private String inicio = "";
	private String inicioComunicado = "";
	private String coordenadasInicio = "";
	private long cuentaKilometrosInicio = 0;
	private String origen = "";
	private String fin = "";
	private String finComunicado = "";
	private String coordenadasFin = "";
	private long cuentaKilometrosFin = 0;
	private String destino = "";
	private String trabajado = "";
	private long tiempo = 0;
	private int kilometros = 0;
	
	@ManyToOne
	@JoinColumn(name = "parte")
	@JsonIgnore
	private ParteJPA parte;

	@OneToMany(cascade = CascadeType.ALL, fetch = FetchType.LAZY)
	@JoinColumn(name = "actividad")
	@OrderBy("id")
	private Set<ListaJPA> listas;

	public ActividadJPA() {
		this.listas = new HashSet<ListaJPA>();
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public TipoActividadJPA getTipo() {
		return tipo;
	}

	public void setTipo(TipoActividadJPA tipo) {
		this.tipo = tipo;
	}

	public Boolean getAbierta() {
		return abierta;
	}

	public void setAbierta(Boolean abierta) {
		this.abierta = abierta;
	}

	public String getTransporte() {
		return transporte;
	}

	public void setTransporte(String transporte) {
		this.transporte = transporte;
	}

	public String getVehiculo() {
		return vehiculo;
	}

	public void setVehiculo(String vehiculo) {
		this.vehiculo = vehiculo;
	}

	public String getRemolque() {
		return remolque;
	}

	public void setRemolque(String remolque) {
		this.remolque = remolque;
	}

	public String getInicio() {
		return inicio;
	}

	public void setInicio(String inicio) {
		this.inicio = inicio;
	}

	public String getInicioComunicado() {
		return inicioComunicado;
	}

	public void setInicioComunicado(String inicioComunicado) {
		this.inicioComunicado = inicioComunicado;
	}

	public String getCoordenadasInicio() {
		return coordenadasInicio;
	}

	public void setCoordenadasInicio(String coordenadasInicio) {
		this.coordenadasInicio = coordenadasInicio;
	}

	public long getCuentaKilometrosInicio() {
		return cuentaKilometrosInicio;
	}

	public void setCuentaKilometrosInicio(long cuentaKilometrosInicio) {
		this.cuentaKilometrosInicio = cuentaKilometrosInicio;
	}

	public String getOrigen() {
		return origen;
	}

	public void setOrigen(String origen) {
		this.origen = origen;
	}

	public String getFin() {
		return fin;
	}

	public void setFin(String fin) {
		this.fin = fin;
		try {
			calcularHoras();
		} catch (ParseException e) {
			e.printStackTrace();
		}
	}

	public String getFinComunicado() {
		return finComunicado;
	}

	public void setFinComunicado(String finComunicado) {
		this.finComunicado = finComunicado;
	}

	public String getCoordenadasFin() {
		return coordenadasFin;
	}

	public void setCoordenadasFin(String coordenadasFin) {
		this.coordenadasFin = coordenadasFin;
	}

	public long getCuentaKilometrosFin() {
		return cuentaKilometrosFin;
	}

	public void setCuentaKilometrosFin(long cuentaKilometrosFin) {
		this.cuentaKilometrosFin = cuentaKilometrosFin;
	}

	public String getDestino() {
		return destino;
	}

	public void setDestino(String destino) {
		this.destino = destino;
	}

	public String getTrabajado() {
		return trabajado;
	}

	public void setTrabajado(String trabajado) {
		this.trabajado = trabajado;
	}

	public long getTiempo() {
		return tiempo;
	}

	public void setTiempo(long tiempo) {
		this.tiempo = tiempo;
	}

	public int getKilometros() {
		return kilometros;
	}

	public void setKilometros(int kilometros) {
		this.kilometros = kilometros;
	}

	public ParteJPA getParte() {
		return parte;
	}

	public void setParte(ParteJPA parte) {
		this.parte = parte;
	}

	public Set<ListaJPA> getListas() {
		return listas;
	}

	public void setListas(Set<ListaJPA> listas) {
		this.listas = listas;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ActividadJPA other = (ActividadJPA) obj;
		if (id != other.id)
			return false;
		return true;
	}
	
	/**
	 * Métode que calcula la diferencia en milisegons des de l'inici al fi de una activitat.
	 * @throws ParseException
	 */
	private void calcularHoras() throws ParseException{
		String inicio = this.getInicio();
		String fin = this.getFin();
		if(!inicio.isEmpty() && !fin.isEmpty()){
			Calendar i = Calendar.getInstance();
			i.setTime(new SimpleDateFormat("dd/MM/yyyy HH:mm").parse(inicio));
			Calendar f = Calendar.getInstance();
			f.setTime(new SimpleDateFormat("dd/MM/yyyy HH:mm").parse(fin));
			long total = f.getTimeInMillis() - i.getTimeInMillis();
			long m = total / 60000;
			long h = m / 60;
			long d = h / 24;
			this.setTiempo(total);
			this.setTrabajado(d + "\tdías\n" + h % 24 + "\thoras\n" + m % 60 + "\tminutos\n");
		}
		
	}
}
