package jpa;

import java.io.Serializable;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

/**
 * Classe que implementa una activitat en format reduït per aprofitar la desserialització JSON automàtica de la llibreria Jackson i actualitzar les entitats amb tan sols una comanda 'merge'.
 * @author Guillem
 *
 */
@Entity
@Table(name = "tfg.actividades")
public class ActividadUpdateJPA implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private int id;

	private String transporte = "";
	private String vehiculo = "";
	private String remolque = "";
	private String inicioComunicado = "";
	private long cuentaKilometrosInicio = 0;
	private String origen = "";
	private String finComunicado = "";
	private long cuentaKilometrosFin = 0;
	private String destino = "";
	private long kilometros = 0;

	public ActividadUpdateJPA() {

	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getTransporte() {
		return transporte;
	}

	public void setTransporte(String transporte) {
		this.transporte = transporte;
	}

	public String getVehiculo() {
		return vehiculo;
	}

	public void setVehiculo(String vehiculo) {
		this.vehiculo = vehiculo;
	}

	public String getRemolque() {
		return remolque;
	}

	public void setRemolque(String remolque) {
		this.remolque = remolque;
	}

	public String getInicioComunicado() {
		return inicioComunicado;
	}

	public void setInicioComunicado(String inicioComunicado) {
		this.inicioComunicado = inicioComunicado;
	}

	public long getCuentaKilometrosInicio() {
		return cuentaKilometrosInicio;
	}

	public void setCuentaKilometrosInicio(long cuentaKilometrosInicio) {
		this.cuentaKilometrosInicio = cuentaKilometrosInicio;
		calcularKilometros();
	}

	public String getOrigen() {
		return origen;
	}

	public void setOrigen(String origen) {
		this.origen = origen;
	}

	public String getFinComunicado() {
		return finComunicado;
	}

	public void setFinComunicado(String finComunicado) {
		this.finComunicado = finComunicado;
	}

	public long getCuentaKilometrosFin() {
		return cuentaKilometrosFin;
	}

	public void setCuentaKilometrosFin(long cuentaKilometrosFin) {
		this.cuentaKilometrosFin = cuentaKilometrosFin;
		calcularKilometros();
	}

	public String getDestino() {
		return destino;
	}

	public void setDestino(String destino) {
		this.destino = destino;
	}

	public long getKilometros() {
		return kilometros;
	}

	public void setKilometros(long kilometros) {
		this.kilometros = kilometros;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ActividadUpdateJPA other = (ActividadUpdateJPA) obj;
		if (id != other.id)
			return false;
		return true;
	}
	
	private void calcularKilometros(){
		long iniciales = this.getCuentaKilometrosInicio();
		long finales = this.getCuentaKilometrosFin();
		if(iniciales > 0 && finales > 0){
			this.setKilometros(finales - iniciales);
		}
	}

}
