package jpa;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;

import com.fasterxml.jackson.annotation.JsonIgnore;

@Entity
@Table(name = "tfg.fases")
public class FaseJPA implements Serializable {

	private static final long serialVersionUID = 1L;
	
	@Id
	@GeneratedValue(strategy=GenerationType.IDENTITY)
	private int id;
	
	private String nombre;
	private int orden;
	private Boolean abierta = true;
	
	@ManyToOne
	@JoinColumn(name = "lista")
	@JsonIgnore
	private ListaJPA lista;
	
	@OneToMany(cascade = CascadeType.ALL, fetch = FetchType.LAZY)
	@JoinColumn(name = "fase")
	@OrderBy("id")
	private Set<RequisitoJPA> requisitos;

	public FaseJPA() {
		this.abierta = true;
		this.requisitos = new HashSet<RequisitoJPA>();
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public int getOrden() {
		return orden;
	}

	public void setOrden(int orden) {
		this.orden = orden;
	}

	public Boolean getAbierta() {
		return abierta;
	}

	public void setAbierta(Boolean abierta) {
		this.abierta = abierta;
	}

	public ListaJPA getLista() {
		return lista;
	}

	public void setLista(ListaJPA lista) {
		this.lista = lista;
	}

	public Set<RequisitoJPA> getRequisitos() {
		return requisitos;
	}

	public void setRequisitos(Set<RequisitoJPA> requisitos) {
		this.requisitos = requisitos;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		FaseJPA other = (FaseJPA) obj;
		if (id != other.id)
			return false;
		return true;
	}

//	@Override
//	public String toString() {
//		return "FaseJPA [id=" + id + ", nombre=" + nombre + ", orden=" + orden + ", abierta=" + abierta
//				+ ", requisitos=" + requisitos + "]";
//	}
	
	@Override
	public String toString() {
		return "FaseJPA [id=" + id + "orden=" + orden + "]";
	}

}
