package jpa;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;

import com.fasterxml.jackson.annotation.JsonIgnore;

@Entity
@Table(name = "tfg.listas")
public class ListaJPA implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private int id;
	
	private String nombre;
	private int maxFases;
	private int tipoLista;
	
	@ManyToOne
	@JoinColumn(name = "actividad")
	@JsonIgnore
	private ActividadJPA actividad;

	@OneToMany(cascade = CascadeType.ALL, fetch = FetchType.LAZY)
	@JoinColumn(name = "lista")
	@OrderBy("id")
	private Set<FaseJPA> fases;

	public ListaJPA() {
		this.fases = new HashSet<FaseJPA>();
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public int getMaxFases() {
		return maxFases;
	}

	public void setMaxFases(int maxFases) {
		this.maxFases = maxFases;
	}

	public int getTipoLista() {
		return tipoLista;
	}

	public void setTipoLista(int tipoLista) {
		this.tipoLista = tipoLista;
	}

	public ActividadJPA getActividad() {
		return actividad;
	}

	public void setActividad(ActividadJPA actividad) {
		this.actividad = actividad;
	}

	public Set<FaseJPA> getFases() {
		return fases;
	}

	public void setFases(Set<FaseJPA> fases) {
		this.fases = fases;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ListaJPA other = (ListaJPA) obj;
		if (id != other.id)
			return false;
		return true;
	}

//	@Override
//	public String toString() {
//		return "ListaJPA [id=" + id + ", nombre=" + nombre + ", maxFases=" + maxFases + ", tipoLista=" + tipoLista
//				+ ", fases=" + fases + "]";
//	}
	
	@Override
	public String toString() {
		return "ListaJPA [fases=" + fases + "]";
	}

}
