package jpa;

import java.io.Serializable;
import java.text.ParseException;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;

import com.fasterxml.jackson.annotation.JsonIgnore;

@Entity
@Table(name = "tfg.partes")
public class ParteJPA implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private int id;

	private Boolean abierto = true;
	private String inicio;
	private String inicioComunicado = "";
	private String coordenadasInicio = "";
	private String fin = "";
	private String finComunicado = "";
	private String coordenadasFin = "";
	private String trabajado = "";
	private long tiempo = 0;
	private long kilometros = 0;

	@ManyToOne(optional = false)
	@JoinColumn(name = "conductor")
	@JsonIgnore
	private UsuarioJPA conductor;

	@OneToMany(cascade = CascadeType.ALL, fetch = FetchType.LAZY)
	@JoinColumn(name = "parte")
	@OrderBy("id")
	private Set<ActividadJPA> actividades;

	public ParteJPA() {
		this.actividades = new HashSet<ActividadJPA>();
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public Boolean getAbierto() {
		return abierto;
	}

	public void setAbierto(Boolean abierto) {
		this.abierto = abierto;
	}

	public String getInicio() {
		return inicio;
	}

	public void setInicio(String inicio) {
		this.inicio = inicio;
	}

	public String getInicioComunicado() {
		return inicioComunicado;
	}

	public void setInicioComunicado(String inicioComunicado) {
		this.inicioComunicado = inicioComunicado;
	}

	public String getCoordenadasInicio() {
		return coordenadasInicio;
	}

	public void setCoordenadasInicio(String coordenadasInicio) {
		this.coordenadasInicio = coordenadasInicio;
	}

	public String getFin() {
		return fin;
	}

	public void setFin(String fin) {
		this.fin = fin;
	}

	public String getFinComunicado() {
		return finComunicado;
	}

	public void setFinComunicado(String finComunicado) {
		this.finComunicado = finComunicado;
	}

	public String getCoordenadasFin() {
		return coordenadasFin;
	}

	public void setCoordenadasFin(String coordenadasFin) {
		this.coordenadasFin = coordenadasFin;
	}

	public String getTrabajado() {
		try {
			calcularTiempo();
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return trabajado;
	}

	public void setTrabajado(String trabajado) {
		this.trabajado = trabajado;
	}

	public long getTiempo() {
		return tiempo;
	}

	public void setTiempo(long tiempo) {
		this.tiempo = tiempo;
	}

	public long getKilometros() {
		calcularKilometros();
		return kilometros;
	}

	public void setKilometros(long kilometros) {
		this.kilometros = kilometros;
	}

	public UsuarioJPA getConductor() {
		return conductor;
	}

	public void setConductor(UsuarioJPA conductor) {
		this.conductor = conductor;
	}

	public Set<ActividadJPA> getActividades() {
		return actividades;
	}

	public void setActividades(Set<ActividadJPA> actividades) {
		this.actividades = actividades;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ParteJPA other = (ParteJPA) obj;
		if (id != other.id)
			return false;
		return true;
	}

//	@Override
//	public String toString() {
//		return "ParteJPA [id=" + id + ", abierto=" + abierto + ", inicio=" + inicio + ", inicioComunicado="
//				+ inicioComunicado + ", coordenadasInicio=" + coordenadasInicio + ", fin=" + fin + ", finComunicado="
//				+ finComunicado + ", coordenadasFin=" + coordenadasFin + ", actividades=" + actividades + "]";
//	}
	
	@Override
	public String toString() {
		return "ParteJPA [actividades=" + actividades + "]";
	}
	
	private void calcularTiempo() throws ParseException{
		Set<ActividadJPA> actividades = this.getActividades();
		long suma = 0;
		for(ActividadJPA actividad : actividades){
			suma += actividad.getTiempo();
		}
		long m = suma / 60000;
		long h = m / 60;
		long d = h / 24;
		this.setTiempo(suma);
		this.setTrabajado(d + "\tdías\n" + h % 24 + "\thoras\n" + m % 60 + "\tminutos\n");
	}
	
	private void calcularKilometros(){
		
		Set<ActividadJPA> actividades = this.getActividades();
		long suma = 0;
		for(ActividadJPA actividad : actividades){
			suma += actividad.getKilometros();
		}
		this.setKilometros(suma);
	}
}
