package managedbean;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import javax.ejb.EJB;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import org.primefaces.model.map.DefaultMapModel;
import org.primefaces.model.map.LatLng;
import org.primefaces.model.map.MapModel;
import org.primefaces.model.map.Marker;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import ejb.DTFacadeLocal;
import jpa.ListaJPA;
import jpa.ParteJPA;
import jpa.UsuarioJPA;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.data.JsonDataSource;

@ManagedBean(name = "upmb")
@ViewScoped
public class UsuariosPartesMBean implements Serializable {

	private static final long serialVersionUID = 1L;
	
	private String REPORT_PATH = "C:\\tfg\\gmartinc\\reports\\parte.jasper";
	private String REPORT_EXPORT_PATH = "C:\\tfg\\gmartinc\\reports";

	@EJB
	private DTFacadeLocal dtLocal;

	private int idUsuario;
	private UsuarioJPA usuario;
	private List<UsuarioJPA> usuarios;
	private int idParte;
	private ParteJPA parte;
	private Set<ParteJPA> partes;
	private ListaJPA lista;
	private String coordenadasSeleccionadas;

	private MapModel mapa = null;

//	@PostConstruct
//	public void init() {
//		String[] coordenadas = coordenadasSeleccionadas.split(",");
//		Double lat = Double.parseDouble(coordenadas[0]);
//		Double lng = Double.parseDouble(coordenadas[1]);
//		LatLng coord = new LatLng(lat, lng);
//		mapa = new DefaultMapModel();
//		mapa.addOverlay(new Marker(coord));
//	}

	public UsuariosPartesMBean() {

	}

	public int getIdUsuario() {
		return idUsuario;
	}

	public void setIdUsuario(int idUsuario) {
		this.idUsuario = idUsuario;
	}

	public UsuarioJPA getUsuario() {

		return usuario;
	}

	public void setUsuario(UsuarioJPA usuario) {
		this.usuario = usuario;
	}

	public List<UsuarioJPA> getUsuarios() throws Exception {
		this.usuariosList();
		return usuarios;
	}

	public void setUsuarios(List<UsuarioJPA> usuarios) {

		this.usuarios = usuarios;
	}

	public int getIdParte() {

		return idParte;
	}

	public void setIdParte(int idParte) {

		this.idParte = idParte;
	}

	public ParteJPA getParte() {
		this.verParte();
		return parte;
	}

	public void setParte(ParteJPA parte) {

		this.parte = parte;
	}

	public Set<ParteJPA> getPartes() throws Exception {

		return partes;
	}

	public void setPartes(Set<ParteJPA> partes) {
		this.partes = partes;
	}

	public ListaJPA getLista() {
		return lista;
	}

	public void setLista(ListaJPA lista) {
		this.lista = lista;
	}

	public String getCoordenadasSeleccionadas() {
		return coordenadasSeleccionadas;
	}

	public void setCoordenadasSeleccionadas(String coordenadasSeleccionadas) {
		this.coordenadasSeleccionadas = coordenadasSeleccionadas;
	}

	public MapModel getMapa() {
		if (coordenadasSeleccionadas != null) {
			String[] coordenadas = coordenadasSeleccionadas.split(",");
			Double lat = Double.parseDouble(coordenadas[0]);
			Double lng = Double.parseDouble(coordenadas[1]);
			LatLng coord = new LatLng(lat, lng);
			mapa = new DefaultMapModel();
			mapa.addOverlay(new Marker(coord));
		}
		return mapa;
	}

	public void setMapa(MapModel mapa) {
		this.mapa = mapa;
	}

	private void usuariosList() throws Exception {
		setUsuarios(dtLocal.verUsuarios());
	}

	private void partesList() throws Exception {
		setPartes(dtLocal.verPartesPorUsuario(idUsuario));
	}

	private void verParte() {
		setParte(dtLocal.verParte(idParte));
	}

	public void onDriverSelect() {
		try {
			partesList();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	public void onPdfClick(ParteJPA parte) throws Exception {
		if (parte != null) {
			String json;
			try {
				json = new ObjectMapper().writeValueAsString(parte);
				dtLocal.generarPdf(json);
				generarPDF(json);
			} catch (JsonProcessingException e) {
				e.printStackTrace();
			}
		}
	}
	
	public void generarPDF(String json) throws IOException {
		HashMap<String, Object> map = new HashMap<>();

		try {
			InputStream is = new ByteArrayInputStream(json.getBytes(StandardCharsets.UTF_8));
			JsonDataSource ds;
			ds = new JsonDataSource(is);
			JasperPrint print = JasperFillManager.fillReport(REPORT_PATH, map, ds);
//			JasperExportManager.exportReportToPdfFile(print, REPORT_EXPORT_PATH + "\\Parte_" + (new SimpleDateFormat("ddMMyyyyHHmmss").format(new Date())) + ".pdf");
			FacesContext context = FacesContext.getCurrentInstance();
			HttpServletResponse response = (HttpServletResponse) context.getExternalContext().getResponse();
			response.setContentType("application/pdf");
			response.setHeader("Content-Disposition", "attachment; filename=Parte_" + (new SimpleDateFormat("ddMMyyyyHHmmss").format(new Date())) + ".pdf");
			final ServletOutputStream outStream = response.getOutputStream();
			JasperExportManager.exportReportToPdfStream(print, outStream);
			outStream.flush();
			outStream.close();
			
		} catch (JRException e) {
			e.printStackTrace();
		}

	}

}
